% % indicates a comment (not evaluated by interpretter)

% This is a script, which can be run directly
% Alternatives to scripts are functions, which are called by scripts (or
% other functions), and cannot be executed directly (mostly).
% My philosophy - put major functions in separate file, and call them from
% a driver script, like this file driver_MATLAB_intro.m.

% Useful to clear all variables and close all plots
% Also clc to clear Command Window
clear all; close all;

% Can define variables with appropriate (and legal) names
var1 = 2.5;
% Basic variable is a vector (array, optimized list holding 1 data type)
v1 = [1;2];
% = is assignment operator, LHS is variable name of your choice
% Note:  semicolon suppresses output
v2 = [1;2;3];

% Indexing used to get entries
% Note:  starts at 1, NOT 0
v2(1);
v2(2);
v2(3);
%v2(4) % error, since v2 has only 2 entries

% Semicolon for rows, comma for column
v3 = [1,2,3];

% Matrices can entered very intuitively
A1 = [1, 2; 0 pi];
% Note:  some built in constants, like pi, exp(1), i

% Standard matrix vector-multiplication
%b1 = A1*v3;
% Won't work, since dimensions aren't correct (A=2x2, v3 = 1x3)
v4 = [1,2];
%b2 = A1*v4;
% Still won't work, since A=2x2, v4=1x2
v5 = v4'; % ' = transpose, so v5=2x1
b3 = A1*v5;  % OKAY!

% Most operations are done pointwise
a1 = [0;5;4];
%v2;
sum1 = a1+v2;

% Multiply scalars by vectors (or matrices)
2*A1;

% Multiplication is, by default, matrix, so pointwise won't make sense
%a1*v2  %a1=3x1, v2=3x1, so matrix product is undefined
% Use .* for poinbtwise multiplication
a1.*v2;
% Same thing for raising to a power (after all, just multiplication)
% a1^2  % Incorrect
%a1.^2;  % Correct

% One of most useful things is plotting a function
% Many ways to do.  My preference is just to numerically do it with a
% vector
% Symbolic techniques also exsit (ezplot, but I will not talk about here)
% Say we want to plot cos(x) on [0,2*pi]
X = 0:0.01:2*pi;  % Define vector from 0-2*pi, with spacing 0.01
f = @(x) cos(x);  % Function handle (inline function, not separate file)
F = f(X);  %Vector, same length as X, where each entry corresponds to f(x), for respective x in X

% Now plot X again X
figure
plot(X,F,'--r','LineWidth',2);  %Line styles indicated with '-k' type command
                               % '-k' means solid line colored black
                               % '-.c' mean dash-dot with color cyan, etc.
xlabel('x');
ylabel('y');
title('f(x) = cos(x)');

% Could plot many things on same set of axes, using hold on;
g = @(x) sin(x); % Plot cos(x) and sin(x) together
G = g(X);

figure
plot(X,F,'-k','LineWidth',2);
hold on;  % Plot something else on top of same figure, instead of replacing
plot(X,G,'--b','LineWidth',2);
legend('cos(x)','sin(x)');  % Add a legend, with label based on order of plot commands and entries to legend function 
xlabel('x');
ylabel('y');
title('Cos(x) and Sin(x)');

% Many other types of plots exist
% Some examples:
% plot3 - 3D plot of surface
% histogram - plotting frequencies of certain events
% scatter - scatter plot of (X,Y) data, that can be colored by a third value
% imagesc - alternative viewing of surface via a heatmap
% quiver - plot a vector field

% Example of quiver - often useful for ODEs in visualization
% Draws arrows for vector field X'=F(X), on some domain (user-specified)
% Example for Lotka-Volterra model (predator-prey)
% Have another file (function) which specifies F(X) for Lotka-Volterra system - Lotka_Volterra.m 
% Define a grid where you would like to visualize the vector field
[X,Y] = meshgrid(0:0.2:5, 0:0.2:4);  % Lattice to evaluate F(X) on
% Sample parameters
a_test = 1;
b_test = 2;
c_test = 0.5;
d_test = 0.25;
% Define dx_dt and dy_dt directly, since MATLAB does not allow chained expressions (i.e. I cannot call Lotka_Volterra(t,[X;Y],a_test,b_test,c_test,d_test)(1) for dx_dt
% Note the use of .* when multiplying (possible) vector quanitities, since I generally want this to be component-wise multiplication
dX_dt = a_test*X - b_test*X.*Y;
dY_dt = c_test*X.*Y - d_test*Y;
% Ideally I would like to use the function Lotka_Volterra.m here, but I cannot because of MATLABs structure (at least, as far as I know)

% Plot the vector field (finally)
figure
quiver(X,Y,dX_dt,dY_dt,'r','LineWidth',2);
xlabel('x (prey)');
ylabel('y (predator)');
title('Vector field for Lotka-Volterra ODEs');
axis([0 max(X(1,:)) 0 max(Y(:,1))]);  % Center the axis on region of interest

% Okay, but length of vectors is making picure hard to understand
% Let's normalize, so all of the vectors are unit in length
L = sqrt(dX_dt.^2+dY_dt.^2);
dX_dt = dX_dt./L;
dY_dt = dY_dt./L;

figure
quiver(X,Y,dX_dt,dY_dt,'r','LineWidth',2);
xlabel('x (prey)');
ylabel('y (predator)');
title('Vector field for Lotka-Volterra ODEs');
axis([0 max(X(1,:)) 0 max(Y(:,1))]);  % Just to automatically detect axis lengths

% Now let's add some trajectories (ODE solutions)
% Let's say we want to solve the ODE (numerically) forward in time for a set of initial conditions
X0 = [0;1;3];   % Initial x (prey)
Y0 = [2;3;1.2]; % Initial y (predator)
% Use a for loop to solve and plot for each set of initial conditions
% Must also specify time interval to solve on
t0 = 0;
tF = 100;
% for loop executes statements repeatedly a fixed number of time, incrementing an index on each iteration
% Here, out for loop will go through indices for pairs of intial conditions
figure  % Call here, since will be plotting on every iteration of loop
for i =1:length(X0)  % X0 or Y0, since have the same length
    % length(vector) returns number of elements (often useful for iterations)
    % i is index from 1,2,...,length(X0)=3 in this case
    % Could increment by every 2nd index (e.g.) using for i=1:2:length(X0)
    initials = [X0(i);Y0(i)];  % Form a pair of initial conditions
    % Solve the ODE using ode45
    [T,XY] = ode45(@(t,X)Lotka_Volterra(t,X,a_test,b_test,c_test,d_test),[t0 tF], initials);
    % T = time vector solver used (T(1)=t0, T(end)=tF)
    % XY = length(T) x 2 matrix, each column represents a component of system
    % Since 2 dimensional, XY has 2 columns
    % Extract individual components (x and y)
    x = XY(:,1);  % First column (: indicates to take all rows here) - rest is just like matrix notation (row,column)
    y = XY(:,2);  % Second column
    plot(x,y,'-k','LineWidth',2);
    hold on;
end  % Must end every construct (except scripts) with end (functions, conditionals, loops, etc.)
% Also plot vector field (quiver) with solution curves
% Same as previous figure
quiver(X,Y,dX_dt,dY_dt,'r','LineWidth',2);
xlabel('x (prey)');
ylabel('y (predator)');
title('Vector field for Lotka-Volterra ODEs');
axis([0 max(X(1,:)) 0 max(Y(:,1))]);  % Just to automatically detect axis lengths

% while loop also exists (similar, but evaluate until statement is False, i.e. not a fixed number of times)

% Also should discuss conditions (if, elseif, else)
% Say I want to use a step-function in some calculation - see step_sample.m
% Just plot in this example
X2 = -2:0.05:10;  % Vector from -2 to 10, with spacing of 0.05
Y2 = zeros(length(X2),1);  % Create a vector of zeros to store Y2 values, which will be evaluation of step_sample on X2
% Use for loop to evaluate at each entry of X2
for j = 1:length(X2)
    Y2(j) = step_sample(X2(j));
end
% Plot results as before
figure
plot(X2,Y2,'+b','LineWidth',2);  % Not connecting dots, but marking each data point with "o", since using LineStyle 'ob'
xlabel('x');
ylabel('f(x)');
title('Step function');

% Other topics
% Matrices
% Matrix-vector multiplication, as well as solving linear systems, is optimized in MATLAB, so use it whenever you can
B1 = zeros(10,8);  % Matrix of zeros of size 10x8 (10 rows, 8 columns)
% Access elements intuitively
B1(2,3);  % 2nd row, 3rd column
B2 = ones(3,3); % Similar, except all ones
B3 = diag(v3);  % Make SQUARE diagonal matrix with v3 along main diagonal (everything else 0)
B4 = B2 + B3;  % Add component-wise (note: defined, since B2 and B3 are same size)
[eig_vecs,eig_vals] = eig(B4);  % Finds eigenvectors and eigenvalues of B4
                                % Each column of eig_vecs is an eigenvector
                                % Each diagonal entry in eig_vals is corresponding eigenvalue
b4 = B4(2,:);  % Get 2nd row of B4, for example
% Many other commands exist (determinant, exp, etc.), and most commands will work on matrices as well
m = mean(B4);  % Mean of each column of B4
total_m = mean(m);  % Get mean of all elements

% Working with external data
% Often want to analyze data produced from external source (e.g. experiment)
% Input and then manipulate using MATLAB
% Of course, how to input depends on KIND of data (image, numbers, strings, etc.)
% And how it is stored (csv, excel, txt, etc.)
% Example of data I have used in the past, stored in csv (comman-separate values)
% Import into vectors
time_data = csvread('time.csv');
control_data = csvread('CONTROL.csv');  % 5 runs of a control (no treatment), and corresponding time in time_data
num_controls = length(control_data(1,:));  % Get number of control experiments directly from data
% Plot controls as before, using a for loop
figure
for ind_control=1:num_controls
    % Here the index (ind_control) is across each experiment (stored in different columns)
    plot(time_data,control_data(:,ind_control),'-r','LineWidth',1);  % Plot all rows, for a given column (by index ind_control)
    hold on;
end
% Also plot mean at each time point
average_response = mean(control_data,2);  % Want means across rows (a given time corresponds to a row), so I have to tell along dimension 2 (rows)
plot(time_data,average_response,'-k','LineWidth',2);
xlabel('time (hours)');
title('In vitro tumor data');

